<?php

/* Reminder: always indent with 4 spaces (no tabs). */
// +---------------------------------------------------------------------------+
// | phpBBBridge Plugin for Geeklog - The Ultimate Weblog                      |
// +---------------------------------------------------------------------------+
// | functions.inc                                                             |
// |                                                                           |
// | This file fulfills a couple purposes.                                     |
// | 1) When the plugin is enabled, this code is included at the end of each   |
// |    lib-common.php load.  This enables me to place phpBB session           |
// |    'keep-alive' code here.                                                |
// | 2) It contains all of the common functions and API hook implementations   |
// |    for this plugin.                                                       |
// +---------------------------------------------------------------------------+
// | Copyright (C) 2004 by the following authors:                              |
// |                                                                           |
// | Turias - http://demo.turias.net/                                          |
// +---------------------------------------------------------------------------+
// |                                                                           |
// | This program is free software; you can redistribute it and/or             |
// | modify it under the terms of the GNU General Public License               |
// | as published by the Free Software Foundation; either version 2            |
// | of the License, or (at your option) any later version.                    |
// |                                                                           |
// | This program is distributed in the hope that it will be useful,           |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of            |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
// | GNU General Public License for more details.                              |
// |                                                                           |
// | You should have received a copy of the GNU General Public License         |
// | along with this program; if not, write to the Free Software Foundation,   |
// | Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.           |
// |                                                                           |
// +---------------------------------------------------------------------------+

// Include the phpBBBridge lang and config files
require_once($_CONF['path'] . 'plugins/phpbbbridge/config.php');
$langfile = $_CONF['path'] . 'plugins/phpbbbridge/language/'
          . $_CONF['language'] . '.php';
if (file_exists ($langfile))
{
    require_once($langfile);
} 
else
{
    include ($_CONF['path'] . 'plugins/phpbbbridge/language/english.php');
}

define('PHPBBBRIDGE_INSTALLED', true);

// If this is loaded before the phpBB common.php file, we assume that this is a
// Geeklog page being loaded.  Thus, we need to handle phpBB sessions.
// Otherwise, we can assume that phpBB will handle them for us.
if (!defined('IN_PHPBB'))
{
    define('IN_GEEKLOG', true);
    
    // Now, we also define that we are in phpBB so that
    // phpBB does not die on us when we load up the phpBB files.
    define('IN_PHPBB', true);

    // We need to save $result somewhere, since phpBB will step on it before
    // Geeklog is finished with it.
    if (isset($result))
    {
        $tmpResult = $result;
    }

    // Load phpBB common.php files
    $phpbb_root_path = $_PHPBBB_CONF['path_to_phpbb'];
    require_once($phpbb_root_path . 'extension.inc');
    require_once($phpbb_root_path . 'common.'.$phpEx);

    if (!defined('PHPBBBRIDGE_DISABLED'))
    {
        //
        // Start session management
        //
        $userdata = session_pagestart($user_ip, PAGE_INDEX);
        init_userprefs($userdata);
        //
        // End session management
        //

        // session id check
        if (!empty($HTTP_POST_VARS['sid']) || !empty($HTTP_GET_VARS['sid']))
        {
            $sid = (!empty($HTTP_POST_VARS['sid'])) ? $HTTP_POST_VARS['sid'] : $HTTP_GET_VARS['sid'];
        }
        else
        {
            $sid = '';
        }
    
        // If for some reason the login is not synched between Geeklog and phpBB... sync it.
        if ($_USER['uid'] > 1 && $_USER['uid'] != $userdata['user_id'])
        {
	    global $HTTP_SERVER_VARS;

            plugin_user_login_phpbbbridge($_USER['uid']);

            if (!isset($HTTP_SERVER_VARS['REQUEST_URI']))
            {
                $arr = explode("/", $HTTP_SERVER_VARS['PHP_SELF']);
                $HTTP_SERVER_VARS['REQUEST_URI'] = "/" . $arr[count($arr)-1];
                if ($HTTP_SERVER_VARS['argv'][0] != "")
                {
                    $HTTP_SERVER_VARS['REQUEST_URI'] .= "?" . $HTTP_SERVER_VARS['argv'][0];
                }
            }

            $server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
            redirect_phpbbbridge($server_protocol . $HTTP_SERVER_VARS['HTTP_HOST'] . $HTTP_SERVER_VARS['REQUEST_URI']);
        }
    }
}

// Reset the $result variable for Geeklog
if (isset($tmpResult))
{
    $result = $tmpResult;
}

// +---------------------------------------------------------------------------+
// | Geeklog Plugin API Implementations                                        |
// +---------------------------------------------------------------------------+

function plugin_chkVersion_phpbbbridge()
{
    global $_PHPBBB_CONF; 
    return $_PHPBBB_CONF['version'];
}

function plugin_getmenuitems_phpbbbridge()
{
    global $_PHPBBB_CONF, $LANG_PHPBBB, $phpEx;
    $menuitems[$LANG_PHPBBB['menulabel']] = $_PHPBBB_CONF['phpbb_url'] . '/index.'.$phpEx;
    return $menuitems;
}

/**
* This will put an option for forum admin in the command and control block on moderation.php
*
*/
function plugin_cclabel_phpbbbridge() 
{
    global $_PHPBBB_CONF, $LANG_PHPBBB, $_USER, $_CONF, $userdata, $phpEx;
    if (SEC_inGroup('Root', $_USER['uid']))
    {
        return array($LANG_PHPBBB['adminmenu'], $_PHPBBB_CONF['phpbb_url'] . "/admin/index.".$phpEx."?sid=".$userdata['session_id'], $_CONF['site_admin_url'] . '/plugins/phpbbbridge/images/phpbbbridge.gif');
    }
}

function plugin_user_login_phpbbbridge($uid)
{
    global $_USER, $db, $board_config, $HTTP_GET_VARS, $HTTP_POST_VARS;
    global $userdata, $user_ip, $session_id, $_PHPBBB_CONF, $phpEx, $_GROUPS;

    // Load phpBB common.php files
    $phpbb_root_path = $_PHPBBB_CONF['path_to_phpbb'];
    require_once($phpbb_root_path . 'extension.inc');
    require_once($phpbb_root_path . 'common.'.$phpEx);

    $username = "";
    if (isset ($HTTP_POST_VARS['loginname'])) {
        $username = COM_applyFilter ($HTTP_POST_VARS['loginname']);
    } else {
        $username = COM_applyFilter ($HTTP_GET_VARS['loginname']);
    }
    if (!$username)
    {
        $username = $_USER['username'];
    }
    
    if( !$userdata['session_logged_in'] )
    {
        $sql = "SELECT user_id, username, user_password, user_active, user_level
                FROM " . USERS_TABLE . "
                WHERE username = '" . str_replace("'", "''", $username) . "'";
        if ( !($result = $db->sql_query($sql)) )
        {
            message_die(CRITICAL_ERROR, "Error obtaining user data during login.", "", __LINE__, __FILE__, $sql);
        }
        if( $row = $db->sql_fetchrow($result) )
        {
            if( $row['user_level'] != ADMIN && $board_config['board_disable'] )
            {
                // The board is disabled.  Do nothing.  The user is not logged into the board but
                // should be redirected to the Geeklog main page, anyway.
            }
            else
            {
		if( $row['user_active'] )
                {
                    $autologin = 0;

                    // We leave admin at 0 since we do not force an additional login for admin pages.  Instead, for
                    // extra security, I just make sure the Geeklog user is in the Root group in phpBB2/admin/pagestart.php
                    $admin = 0;
                    $session_id = session_begin($row['user_id'], $user_ip, PAGE_INDEX, FALSE, $autologin, $admin);

                    if( $session_id )
                    {
                        // We are successfully logged in!
                        $sql = "DELETE FROM " . SESSIONS_TABLE . " WHERE session_user_id = -1 AND "
                             . "session_ip = '" . $user_ip . "'";
                        if ( !$db->sql_query($sql) )
                        {
                            message_die(CRITICAL_ERROR, 'Error clearing sessions table', '', __LINE__, __FILE__, $sql);
                        }
                    }
                    else
                    {
                        die("Could not start phpBB session.");
                    }
                }
                else
                {
                    // User is not active.		    
                }
            }
        }
        else
        {
	    COM_errorLog('User ' . $username . ' does not exist in the phpBB database. Try resyncing your tables from the phpBBBridge installation page.', 1);
        }
    }
    else
    {
        // Already logged in.
    }
}

function plugin_user_logout_phpbbbridge($uid)
{
    global $HTTP_GET_VARS, $HTTP_POST_VARS, $userdata;

    session_end($userdata['session_id'], $uid);
}

/**
* Geeklog is checking to see if this plugin supports comments, tell it no.
*/
function plugin_commentsupport_phpbbbridge() 
{
    return false;
}

function plugin_uninstall_phpbbbridge()
{
    global $_TABLES;
    DB_query("DELETE FROM {$_TABLES['plugins']} WHERE pi_name = 'phpbbbridge'");
}

function plugin_user_delete_phpbbbridge($uid)
{
    global $db;

    $user_id = $uid;
    $this_userdata = get_userdata($user_id);

    $sql = "SELECT g.group_id
            FROM " . USER_GROUP_TABLE . " ug, " . GROUPS_TABLE . " g
            WHERE ug.user_id = $user_id
            AND g.group_id = ug.group_id
            AND g.group_single_user = 1";
    if( !($result = $db->sql_query($sql)) )
    {
        $msg = 'Could not obtain phpBB group information for user ' . $uid;
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $row = $db->sql_fetchrow($result);

    $sql = "UPDATE " . POSTS_TABLE . "
            SET poster_id = " . DELETED . ", post_username = '$username'
            WHERE poster_id = $user_id";
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not update phpBB posts for user ' . $uid;
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "UPDATE " . TOPICS_TABLE . "
            SET topic_poster = " . DELETED . "
            WHERE topic_poster = $user_id";
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not update phpBB topics for user ' . $uid;
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "UPDATE " . VOTE_USERS_TABLE . "
            SET vote_user_id = " . DELETED . "
            WHERE vote_user_id = $user_id";
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not update phpBB votes for user ' . $uid;
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "SELECT group_id
            FROM " . GROUPS_TABLE . "
            WHERE group_moderator = $user_id";
    if( !($result = $db->sql_query($sql)) )
    {
        COM_errorLog('Could not select phpBB groups where user ' . $uid . ' was moderator');
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    while ( $row_group = $db->sql_fetchrow($result) )
    {
        $group_moderator[] = $row_group['group_id'];
    }

    if ( count($group_moderator) )
    {
        $update_moderator_id = implode(', ', $group_moderator);

        $sql = "UPDATE " . GROUPS_TABLE . "
                SET group_moderator = " . $userdata['user_id'] . "
                WHERE group_moderator IN ($update_moderator_id)";
        if( !$db->sql_query($sql) )
        {
            $msg = 'Could not update phpBB group moderators when deleting user ' . $uid;
            COM_errorLog($msg);
            message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
        }
    }

    $sql = "DELETE FROM " . USERS_TABLE . "
            WHERE user_id = $user_id";
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not delete user ' . $uid . ' from the phpBB users table';
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "DELETE FROM " . USER_GROUP_TABLE . "
            WHERE user_id = $user_id";
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not delete user ' . $uid . ' from phpBB user_group table';
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "DELETE FROM " . GROUPS_TABLE . "
            WHERE group_id = " . $row['group_id'];
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not delete user ' . $uid . ' from phpBB groups table';
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "DELETE FROM " . AUTH_ACCESS_TABLE . "
            WHERE group_id = " . $row['group_id'];
    if( !$db->sql_query($sql) )
    {
        $msg = 'Could not delete phpBB auth group for user ' . $uid;
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "DELETE FROM " . TOPICS_WATCH_TABLE . "
            WHERE user_id = $user_id";
    if ( !$db->sql_query($sql) )
    {
        $msg = 'Could not delete user ' . $uid . ' from phpBB topic watch table';
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "DELETE FROM " . BANLIST_TABLE . "
            WHERE ban_userid = $user_id";
    if ( !$db->sql_query($sql) )
    {
        $msg = 'Could not delete user ' . $uid . ' from phpBB banlist table';
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    $sql = "SELECT privmsgs_id
            FROM " . PRIVMSGS_TABLE . "
            WHERE privmsgs_from_userid = $user_id
            OR privmsgs_to_userid = $user_id";
    if ( !($result = $db->sql_query($sql)) )
    {
        $msg = 'Could not select all phpBB private messages of user ' . $uid;
        COM_errorLog($msg);
        message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
    }

    // This little bit of code directly from the private messaging section.
    while ( $row_privmsgs = $db->sql_fetchrow($result) )
    {
        $mark_list[] = $row_privmsgs['privmsgs_id'];
    }

    if ( count($mark_list) )
    {
        $delete_sql_id = implode(', ', $mark_list);

        $delete_text_sql = "DELETE FROM " . PRIVMSGS_TEXT_TABLE . "
                            WHERE privmsgs_text_id IN ($delete_sql_id)";
        $delete_sql = "DELETE FROM " . PRIVMSGS_TABLE . "
                       WHERE privmsgs_id IN ($delete_sql_id)";

        if ( !$db->sql_query($delete_sql) )
        {
            $msg = 'Could not delete phpBB private message info for user ' . $uid;
            COM_errorLog($msg);
            message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
        }

        if ( !$db->sql_query($delete_text_sql) )
        {
            $msg = 'Could not delete phpBB private message text for user ' . $uid;
            COM_errorLog($msg);
            message_die(CRITICAL_ERROR, $msg, "", __LINE__, __FILE__, $sql);
        }
    }
}

function plugin_user_create_phpbbbridge($uid)
{
    global $_TABLES, $db, $board_config;

    $sql = "SELECT * FROM {$_TABLES['users']} u, {$_TABLES['userprefs']} p WHERE u.uid = p.uid AND u.uid = $uid";
    $result = DB_query($sql);
    $A = DB_fetchArray($result);

    $regdate = $A['regdate'];
    $regyear = substr($regdate, 0, 4);
    $regmonth = substr($regdate, 5, 7);
    $regday = substr($regdate, 8, 10);
    $reghour = substr($regdate, 11, 13);
    $regminute = substr($regdate, 14, 16);
    $regsecond = substr($regdate, 17);
    $newreg = mktime($reghour, $regminute, $regsecond, $regmonth, $regday, $regyear);

    $user_level = 0;
    if (SEC_inGroup('Root', $uid))
    {
        $user_level = 1;
    }

    $sql = "INSERT into " . USERS_TABLE
	    . " (user_id, username, user_regdate, user_password, user_email, user_icq, user_website,"
            . " user_occ, user_from, user_interests, user_sig, user_sig_bbcode_uid, user_avatar,"
            . " user_avatar_type, user_viewemail, user_aim, user_yim, user_msnm, user_attachsig,"
            . " user_allowsmile, user_allowhtml, user_allowbbcode, user_allow_viewonline,"
            . " user_notify, user_notify_pm, user_popup_pm, user_timezone, user_dateformat,"
            . " user_lang, user_style, user_level, user_allow_pm, user_active, user_actkey) VALUES ("
            . " " . $A['uid'] . ","
            . " '" . str_replace("'", "''", $A['username']) . "',"
            . " " . $newreg . ","
            . " '" . $A['passwd'] . "',"
	    . " '" . $A['email'] . "',"
            . " '',"
            . " '" . str_replace("'", "''", $A['homepage']) . "',"
            . " '', '', '', '', '', '', 0, 0, '', '', '',"
            . " " . $board_config['allow_sig'] . ","
            . " " . $board_config['allow_smilies'] . ","
            . " " . $board_config['allow_html'] . ","
            . " " . $board_config['allow_bbcode'] . ","
            . " " . $A['showonline'] . ","
            . " 0, 0, 1,"
            . " '" . $board_config['board_timezone'] . "',"
            . " '" . $board_config['default_dateformat'] . "',"
            . " '" . $board_config['default_lang'] . "',"
            . " " . $board_config['default_style'] . ","
            . " " . $user_level . ","
            . " 1,"
            . " 1,"
            . " '')";

    if (!($result = $db->sql_query($sql)))
    {
        COM_errorLog("Error adding " . $A['username'] . " to the phpBB database.", 1);
        message_die(CRITICAL_ERROR, "Error adding user to database.", "", __LINE__, __FILE__, $sql);
    }

    $sql = "INSERT INTO " . GROUPS_TABLE . " (group_name, group_description, group_single_user, group_moderator)
            VALUES ('', 'Personal User', 1, 0)";
    if ( !($result = $db->sql_query($sql)) )
    {
        COM_errorLog("Error adding " . $A['username'] . " to the phpBB database.", 1);
        message_die(CRITICAL_ERROR, "Error adding user to database.", "", __LINE__, __FILE__, $sql);
    }

    $group_id = $db->sql_nextid();
    $user_id = $A['uid'];

    $sql = "INSERT INTO " . USER_GROUP_TABLE . " (user_id, group_id, user_pending)
            VALUES ($user_id, $group_id, 0)";
    if( !($result = $db->sql_query($sql, END_TRANSACTION)) )
    {
        COM_errorLog("Error adding " . $A['username'] . " to the phpBB database.", 1);
        message_die(CRITICAL_ERROR, "Error adding user to database.", "", __LINE__, __FILE__, $sql);
    }
}

function plugin_user_changed_phpbbbridge($uid)
{
    global $_TABLES, $db, $HTTP_POST_VARS;

    $sql = "SELECT * FROM {$_TABLES['users']} u, {$_TABLES['userprefs']} p, {$_TABLES['userinfo']} i WHERE u.uid = p.uid AND u.uid = i.uid AND u.uid = $uid";
    $result = DB_query($sql);
    $nrows = DB_numRows($result);
    if ($nrows == 0)
    {
        // no such user
        return;
    }
    $A = DB_fetchArray($result);

    $showonline = $A['showonline'];
    if (!$showonline)
    {
        $showonline = 1;
    }

    $sql = "UPDATE " . USERS_TABLE . " SET username = '" . str_replace("'", "''", $A['username']) . "'"
	 . ", user_password = '" . $A['passwd'] . "'"
         . ", user_email = '" . $A['email'] . "'"
         . ", user_website = '" . str_replace("'", "''", $A['homepage']) . "'"
         . ", user_from = '" . str_replace("'", "''", $A['location']) . "'"
         . ", user_allow_viewonline = " . $A['showonline'];

    $user_level = 0;

    if (SEC_inGroup('Root') && is_array($HTTP_POST_VARS[$_TABLES['groups']]))
    {
        $rootgrp = DB_getItem ($_TABLES['groups'], 'grp_id', "grp_name = 'Root'");
        if (in_array ($rootgrp, $HTTP_POST_VARS[$_TABLES['groups']]))
        {
            $user_level = 1;
        }
    }
    else if (SEC_inGroup('Root', $uid))
    {
        $user_level = 1;
    }

    // Determine if this user is a Moderator
    if ($user_level != 1)
    {
        $sql2 = "SELECT COUNT(auth_mod) AS is_auth_mod
                 FROM " . AUTH_ACCESS_TABLE . " aa, " . USER_GROUP_TABLE . " ug
                 WHERE ug.user_id = " . $uid . "
                 AND aa.group_id = ug.group_id
                 AND aa.auth_mod = 1";

        if ( !($result2 = $db->sql_query($sql2)) )
        {
	    message_die(GENERAL_ERROR, 'Could not obtain moderator status', '', __LINE__, __FILE__, $sql2);
        }

        if ( !($row2 = $db->sql_fetchrow($result2)) )
        {
            $user_level = 0;
        }
        else
        {
            if ($row2['is_auth_mod'] == 0)
            {
                $user_level = 0;
            }
            else
            {
                $user_level = 2;
            }
        }
    }

    $sql .= ", user_level = " . $user_level
          . " WHERE user_id = " . $uid;

    if ( !($result = $db->sql_query($sql)) )
    {
        COM_errorLog("Error syncing phpBB profile of user " . $uid . " with their Geeklog profile.", 1);
        message_die(CRITICAL_ERROR, "Error syncing phpBB profile of user.", "", __LINE__, __FILE__, $sql);
    }

    return;
}

function plugin_getuseroption_phpbbbridge()
{
    global $_PHPBBB_CONF, $LANG_PHPBBB, $userdata, $lang, $db, $phpEx;

    if ( $userdata['user_new_privmsg'] )
    {
        $l_message = $LANG_PHPBBB['pms'] . ' (' . $userdata['user_new_privmsg'] . ' ' . $LANG_PHPBBB['new'] . ')';

        if ( $userdata['user_last_privmsg'] > $userdata['user_lastvisit'] )
        {
            $sql = "UPDATE " . USERS_TABLE . "
                    SET user_last_privmsg = " . $userdata['user_lastvisit'] . "
                    WHERE user_id = " . $userdata['user_id'];
            if ( !$db->sql_query($sql) )
            {
                message_die(GENERAL_ERROR, 'Could not update private message new/read time for user', '', __LINE__, __FILE__, $sql);
            }

            $s_privmsg_new = 1;
        }
        else
        {
            $s_privmsg_new = 0;
        }
    }
    else if ( $userdata['user_unread_privmsg'] )
    {
        $l_message = $LANG_PHPBBB['pms'] . ' (' . $userdata['user_unread_privmsg'] . ' ' . $LANG_PHPBBB['unread'] . ')';
    }
    else
    {
        $l_message = $LANG_PHPBBB['private_messages'];
    }
    
    return array($l_message, $_PHPBBB_CONF['phpbb_url'] . '/privmsg.'.$phpEx.'?folder=inbox', 0);
} 

function plugin_getadminoption_phpbbbridge()
{
    global $_PHPBBB_CONF, $LANG_PHPBBB, $_USER, $phpEx, $userdata;

    if (SEC_inGroup('Root', $_USER['uid']))
    {
        return array($LANG_PHPBBB['forum_admin_panel'], $_PHPBBB_CONF['phpbb_url'] . '/admin/index.'.$phpEx.'?sid='.$userdata['session_id'], 0);
    }
}

// Mostly yanked from phpBB
function redirect_phpbbbridge($url)
{
	global $db, $board_config;

	if (!empty($db))
	{
		$db->sql_close();
	}

	if (strstr(urldecode($url), "\n") || strstr(urldecode($url), "\r"))
	{
		message_die(GENERAL_ERROR, 'Tried to redirect to potentially insecure url.');
	}

	// Redirect via an HTML form for PITA webservers
	if (@preg_match('/Microsoft|WebSTAR|Xitami/', getenv('SERVER_SOFTWARE')))
	{
		header('Refresh: 0; URL=' . $server_protocol . $server_name . $server_port . $script_name . $url);
		echo '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"><html><head><meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1"><meta http-equiv="refresh" content="0; url=' . $server_protocol . $server_name . $server_port . $script_name . $url . '"><title>Redirect</title></head><body><div align="center">If your browser does not support meta redirection please click <a href="' . $server_protocol . $server_name . $server_port . $script_name . $url . '">HERE</a> to be redirected</div></body></html>';
		exit;
	}

	// Behave as per HTTP/1.1 spec for others
	header('Location: ' . $url);
	exit;
}

function phpblock_phpbbbridge_newtopics()
{
    return show_phpbbbridge_newposts_block(true);
}

function plugin_centerblock_phpbbbridge($where = 1, $page = 1, $topic = '')
{
    global $_CONF, $_USER, $_TABLES, $_PHPBBB_CONF;

    if (!$_PHPBBB_CONF['show_centerblock'])
    {
	return '';
    }

    $retval = '';
    $cb_enable = true;
    $cb_where = 1; // Top of page
    if ($_PHPBBB_CONF['centerblock_after_featured'])
    {
	$cb_where = 2; // After featured story
    }

    if ($_PHPBBB_CONF['centerblock_page_bottom'])
    {
        $cb_where = 3;
    }

    // If enabled only for homepage and this is not page 1 or a topic page,
    // then set disable flag
    if ($_PHPBBB_CONF['centerblock_homepage_only'] && ($page > 1 OR !empty ($topic)))
    {
        $cb_enable = false;
    }
    else if (!$_PHPBBB_CONF['centerblock_homepage_only'] && $page > 1)
    {
        $cb_where = 1;      // Top of Page
    }
    
    if ($cb_where == 2)
    {
        // Check if there are no featured articles in this topic
        // and if so then place it at the top of the page
        if (!empty ($topic))
        {
            $wheresql = "WHERE tid='$topic' AND featured > 0";
        }
        else
        {
            $wheresql = 'WHERE featured = 1';
        }
        $result = DB_query("SELECT sid, owner_id, group_id, perm_owner, perm_group, perm_members, perm_anon FROM {$_TABLES['stories']} $wheresql AND draft_flag=0");
	$nrows = DB_numRows($result);
        if ($nrows == 0)
        {
            $cb_where = 1;
        }
        else
        {
            $A = DB_fetchArray($result);
            if (!SEC_hasAccess($A['owner_id'], $A['group_id'], $A['perm_owner'], $A['perm_group'], $A['perm_members'], $A['perm_anon']))
            {
                $cb_where = 1;
            }
        }
    }

    if ($cb_enable AND ($cb_where == $where))
    {
        $retval .= show_phpbbbridge_newposts_block(false);
    }

    return $retval;
}

function show_phpbbbridge_newposts_block($isSideBlock)
{
    global $_PHPBBB_CONF, $_TABLES, $_USER, $userdata, $_CONF, $db, $LANG_PHPBBB, $board_config, $phpEx;

    $retval = '';

    if ($isSideBlock)
    {
    	$post_limit = $_PHPBBB_CONF['sideblock_num_topics'];
    	$subject_limit = $_PHPBBB_CONF['sideblock_subject_limit'];
    }
    else
    {
    	$post_limit = $_PHPBBB_CONF['centerblock_num_topics'];
    	$subject_limit = $_PHPBBB_CONF['centerblock_subject_limit'];
    }

    $pi_name = 'phpbbbridge';
    if (DB_getItem($_TABLES['plugins'],'pi_enabled', 'pi_name = "' .$pi_name. '"') && defined("IN_PHPBB"))
    {
	$sql = "SELECT DISTINCT topic_id FROM " . POSTS_TABLE;
	$orderby = " ORDER BY post_id DESC LIMIT " . $post_limit;

	//
	// If user is logged in then we'll check to see which (if any) private
	// forums they are allowed to view and include them in the search.
	//
	// If not logged in we explicitly prevent searching of private forums
	//
	$auth_sql = '';
	$is_auth_ary = auth(AUTH_READ, AUTH_LIST_ALL, $userdata); 

	$ignore_forum_sql = '';
	while( list($key, $value) = each($is_auth_ary) )
	{
		if ( !$value['auth_read'] )
		{
			$ignore_forum_sql .= ( ( $ignore_forum_sql != '' ) ? ', ' : '' ) . $key;
		}
	}

	if ( $ignore_forum_sql != '' )
	{
		$auth_sql .= " forum_id NOT IN ($ignore_forum_sql)";
	}

	if ( $auth_sql != '' )
	{
		$sql = $sql . " WHERE" . $auth_sql;
	}
	$sql .= $orderby;

	if ( !($postsResult = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not obtain topic ids', '', __LINE__, __FILE__, $sql);
	}

	$search_ids = array();
	for ($i = 0; $i < $post_limit; $i++)
	{
		if (!($postsRow = $db->sql_fetchrow($postsResult)))
		{
			$i = $post_limit;
		}
		else
		{
			$search_ids[$i] = $postsRow['topic_id'];
		}
	}
	$db->sql_freeresult($postsResult);

	if (count($search_ids) > 0)
	{
	       	$block_header = new Template($_CONF['path_layout'] . 'phpbbbridge/templates');
		$filename = '';
		if ($isSideBlock)
		{
			$filename = 'sideblock_header.thtml';
		}
		else
		{
			$filename = 'centerblock_header.thtml';
		}
	        $block_header->set_file(array('block_header' => $filename));
		$block_header->set_var('LAYOUT_URL', $_CONF['layout_url']);
		$block_header->set_var('BLOCK_HEADER', COM_startBlock($LANG_PHPBBB['latest_forum_posts']));
		$block_header->set_var('LANG_FORUM', $LANG_PHPBBB['forum']);
		$block_header->set_var('LANG_SUBJECT', $LANG_PHPBBB['subject']);
		$block_header->set_var('LANG_DATE', $LANG_PHPBBB['date']);
		$block_header->set_var('LANG_BY', $LANG_PHPBBB['by']);


        	$block_header->parse ('output', 'block_header');
	        $retval .= $block_header->finish($block_header->get_var('output'));

		$i = 0;
		foreach ($search_ids as $topic_id)
		{
			$sql = "SELECT * FROM " . TOPICS_TABLE . " t, " . USERS_TABLE . " u WHERE t.topic_id=" . $topic_id . " AND u.user_id = t.topic_poster";
		        if (!($result = $db->sql_query($sql)))
		        {
		            message_die(CRITICAL_ERROR, "Error obtaining new topic info for sideblock.", "", __LINE__, __FILE__, $sql);
        		}
		        if (!($row = $db->sql_fetchrow($result)))
		        {
	        	    message_die(CRITICAL_ERROR, "Error obtaining new topic info for sideblock.", "", __LINE__, __FILE__, $sql);
			}

			$last_post_id = $row['topic_last_post_id'];

			$author = '';

			$sql2 = "SELECT * FROM " . POSTS_TABLE . " p, " . USERS_TABLE . " u WHERE post_id=" . $last_post_id . " AND u.user_id = p.poster_id";
		        if (!($result2 = $db->sql_query($sql2)))
		        {
		            message_die(CRITICAL_ERROR, "Error obtaining new topic info for sideblock.", "", __LINE__, __FILE__, $sql2);
        		}
		        if (!($row2 = $db->sql_fetchrow($result2)))
		        {
		            message_die(CRITICAL_ERROR, "Error obtaining new topic info for sideblock.", "", __LINE__, __FILE__, $sql2);
			}

			$authorId = $row2['poster_id'];
			if ($authorId == -1)
			{
				$author = $row2['post_username'];
			}
			else
			{
				$author = $row2['username'];
			}
		
	        	$block_displayline = new Template($_CONF['path_layout'] . 'phpbbbridge/templates');
			if ($isSideBlock)
			{
				$filename = 'sideblock_displayline.thtml';
			}
			else
			{
				$filename = 'centerblock_displayline.thtml';
			}
        	        $block_displayline->set_file(array('block_displayline' => $filename));
			$block_displayline->set_var('LAYOUT_URL', $_CONF['layout_url']);
		
			if ($i == 0)
			{
				if ($isSideBlock)
				{
					$block_displayline->set_var('ROW_CLASS', "phpbbsideblock1");
				}
				else
				{
					$block_displayline->set_var('ROW_CLASS', "phpbbcenterblock1");
				}
			}
			else
			{
				if ($isSideBlock)
				{
					$block_displayline->set_var('ROW_CLASS', "phpbbsideblock2");
				}
				else
				{
					$block_displayline->set_var('ROW_CLASS', "phpbbcenterblock2");
				}
			}

			$block_displayline->set_var('TOPIC_URL', $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.'.$phpEx.'?t=' . $topic_id);
			$block_displayline->set_var('LANG_BY_LABEL', $LANG_PHPBBB['by_label']);
			$block_displayline->set_var('LANG_AT_LABEL', $LANG_PHPBBB['at_label']);

			$authorStr = '';
			if ($authorId == -1)
			{
				$authorStr = $author;
			}
			else
			{
				$authorStr = '<a href="' . $_PHPBBB_CONF['phpbb_url'] . '/profile.'.$phpEx.'?mode=viewprofile&amp;u=' . $row2['user_id'] . '">' . $author . '</a>';
			}

			$block_displayline->set_var('AUTHOR', $authorStr);
			$block_displayline->set_var('AUTHOR_NAME', $author);
			$block_displayline->set_var('AUTHOR_PROFILE_URL', $_PHPBBB_CONF['phpbb_url'] . '/profile.'.$phpEx.'?mode=viewprofile&amp;u=' . $row2['user_id']);

			$post_time = create_date($board_config['default_dateformat'], $row2['post_time'], $board_config['board_timezone']);
			$block_displayline->set_var('DATE', $post_time);
			$block_displayline->set_var('LATEST_URL', $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.'.$phpEx.'?p=' . $last_post_id . "#" . $last_post_id);

			$subject = $row['topic_title'];
			if (strlen($subject) > $subject_limit)
			{
				$subject = substr($subject, 0, $subject_limit) . "...";
			}	
			$block_displayline->set_var('SUBJECT', $subject);

			$forumId = $row['forum_id'];
			$sql = "SELECT forum_name FROM " . FORUMS_TABLE . " WHERE forum_id=" . $forumId;
			if (!($result3 = $db->sql_query($sql)))
                	{
	       	            message_die(CRITICAL_ERROR, "Error obtaining forumname for forum.", "", __LINE__, __FILE__, $sql);
        	       	}
                	if (!($row3 = $db->sql_fetchrow($result3)))
	       	        {
       		            message_die(CRITICAL_ERROR, "Error obtaining forumname for forum.", "", __LINE__, __FILE__, $sql);
                	}
			$block_displayline->set_var('FORUM', $row3['forum_name']);
			$block_displayline->set_var('FORUM_URL', $_PHPBBB_CONF['phpbb_url'] . "/viewforum.".$phpEx."?f=" . $forumId);

	                $block_displayline->parse ('output', 'block_displayline');
        	        $retval .= $block_displayline->finish($block_displayline->get_var('output'));

			$i = ($i + 1) % 2;
        	}

	       	$block_footer = new Template($_CONF['path_layout'] . 'phpbbbridge/templates');
		$filename = '';
		if ($isSideBlock)
		{
			$filename = 'sideblock_footer.thtml';
		}
		else
		{
			$filename = 'centerblock_footer.thtml';
		}
	        $block_footer->set_file(array('block_footer' => $filename));
		$block_footer->set_var('LAYOUT_URL', $_CONF['layout_url']);
		$block_footer->set_var('BLOCK_FOOTER', COM_endBlock());

        	$block_footer->parse ('output', 'block_footer');
	        $retval .= $block_footer->finish($block_footer->get_var('output'));
	}

        return $retval;
    } 
    else
    {
        return "";
    }
}

/**
* shows the statistics for the phpBBBridge on stats.php
* If $showsitestats is 1 then we are to only print the overall stats in the 'site statistics box'
* otherwise we show the detailed stats for the plugin
*
* @showsitestate        int         Flag to let us know which stats to get
*/
function plugin_showstats_phpbbbridge($showsitestats) 
{
    global $_CONF, $_TABLES, $db, $LANG_PHPBBB, $_PHPBBB_CONF, $userdata, $phpEx;

    $retval='';

    if ($showsitestats == 1)
    {
        // This shows in the summary box
        $total_posts = DB_count(POSTS_TABLE);
	$total_topics = DB_count(TOPICS_TABLE);
        $summary_label = $LANG_PHPBBB['num_topics_posts_in_forum'];        // Label to display
        
        $retval = "<table border = '0' width='100%' cellspacing='0' cellpadding='0'>";
        $retval .= "<tr><td>$summary_label</td>";
        $retval .= "<td align='right'>" . $total_topics . " (" . $total_posts . ")&nbsp;&nbsp;</td></tr></table>";
    } 
    else
    {
	$sql = '';
        $basesql = "SELECT topic_id, topic_title, topic_views, topic_replies from " . TOPICS_TABLE;
	$limit = " LIMIT 10";

	//
	// If user is logged in then we'll check to see which (if any) private
	// forums they are allowed to view and include them in the search.
	//
	// If not logged in we explicitly prevent searching of private forums
	//
	$auth_sql = '';
	$is_auth_ary = auth(AUTH_READ, AUTH_LIST_ALL, $userdata); 
	$ignore_forum_sql = '';
	while( list($key, $value) = each($is_auth_ary) )
	{
		if ( !$value['auth_read'] )
		{
			$ignore_forum_sql .= ( ( $ignore_forum_sql != '' ) ? ', ' : '' ) . $key;
		}
	}
	if ( $ignore_forum_sql != '' )
	{
		$auth_sql .= " forum_id NOT IN ($ignore_forum_sql)";
	}

	$orderby = " ORDER BY topic_views DESC";
	if ( $auth_sql != '' )
	{
		$sql = $basesql . " WHERE" . $auth_sql . $orderby . $limit;
	}
	else
	{
		$sql = $basesql . $orderby . $limit;
	}

        if (!($result = $db->sql_query($sql)))
        {
            message_die(CRITICAL_ERROR, "Error obtaining most viewed topics.", "", __LINE__, __FILE__, $sql);
       	}

        $retval .= COM_startBlock($LANG_PHPBBB['statsheadingviews']);
        $stat_templates = new Template($_CONF['path_layout'] . 'stats');
        $stat_templates->set_file(array('itemstats'=>'itemstatistics.thtml', 'statrow'=>'singlestat.thtml'));
        $stat_templates->set_var('item_label', $LANG_PHPBBB['subject']);
        $stat_templates->set_var('stat_name', $LANG_PHPBBB['views']);
	while ($row = $db->sql_fetchrow($result))
	{
		$stat_templates->set_var('item_text', $row['topic_title']);
		$stat_templates->set_var('item_stat', $row['topic_views']);
		$stat_templates->set_var('item_url', $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.'.$phpEx.'?t=' . $row['topic_id']);
		$stat_templates->parse('stat_row','statrow',true);
        }
        $stat_templates->parse('output','itemstats');
        $retval .= $stat_templates->finish($stat_templates->get_var('output'));
        $retval .= COM_endBlock();

	$orderby = " ORDER BY topic_replies DESC";
	if ( $auth_sql != '' )
	{
		$sql = $basesql . " WHERE" . $auth_sql . $orderby . $limit;
	}
	else
	{
		$sql = $basesql . $orderby . $limit;
	}

        if (!($result = $db->sql_query($sql)))
        {
            message_die(CRITICAL_ERROR, "Error obtaining most active topics.", "", __LINE__, __FILE__, $sql);
       	}

        $retval .= COM_startBlock($LANG_PHPBBB['statsheadingreplies']);
        $stat_templates = new Template($_CONF['path_layout'] . 'stats');
        $stat_templates->set_file(array('itemstats'=>'itemstatistics.thtml', 'statrow'=>'singlestat.thtml'));
        $stat_templates->set_var('item_label', $LANG_PHPBBB['subject']);
        $stat_templates->set_var('stat_name', $LANG_PHPBBB['replies']);
	while ($row = $db->sql_fetchrow($result))
	{
		$stat_templates->set_var('item_text', $row['topic_title']);
		$stat_templates->set_var('item_stat', $row['topic_replies']);
		$stat_templates->set_var('item_url', $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.'.$phpEx.'?t=' . $row['topic_id']);
		$stat_templates->parse('stat_row','statrow',true);
        }
        $stat_templates->parse('output','itemstats');
        $retval .= $stat_templates->finish($stat_templates->get_var('output'));
        $retval .= COM_endBlock();
    }    
    
    return $retval;
}

function plugin_autotags_phpbbbridge($op, $content='', $autotag='')
{
    global $_PHPBBB_CONF, $phpEx;

    if ($op == 'tagname' )
    {
        return array('forum', 'topic', 'post');
    }
    else if ($op == 'parse')
    {
        if ($autotag['tag'] == 'topic')
        {
		$forumlink = '<a href="' . $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.' . $phpEx . '?t=' . $autotag['parm1'] . '">' . $autotag['parm2'] . '</a>';
        	$content = str_replace($autotag['tagstr'], $forumlink, $content);
        	return $content;
	}
        else if ($autotag['tag'] == 'forum')
        {
		$forumlink = '<a href="' . $_PHPBBB_CONF['phpbb_url'] . '/viewforum.' . $phpEx . '?f=' . $autotag['parm1'] . '">' . $autotag['parm2'] . '</a>';
        	$content = str_replace($autotag['tagstr'], $forumlink, $content);
        	return $content;
	}
        else if ($autotag['tag'] == 'post')
        {
		$forumlink = '<a href="' . $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.' . $phpEx . '?p=' . $autotag['parm1'] . '#' . $autotag['parm1'] . '">' . $autotag['parm2'] . '</a>';
        	$content = str_replace($autotag['tagstr'], $forumlink, $content);
        	return $content;
	}
    }
}

function plugin_profilevariablesdisplay_phpbbbridge($uid, &$template)
{
	global $_TABLES, $_CONF, $LANG_PHPBBB, $_PHPBBB_CONF, $userdata, $board_config, $phpEx, $db;

	$basesql = "SELECT post_time, p.forum_id, post_id, topic_title FROM " . POSTS_TABLE . " p, " . TOPICS_TABLE . " t WHERE t.topic_id=p.topic_id AND p.poster_id=" . $uid;
	$orderby = " ORDER BY post_time DESC LIMIT 10";

	//
	// If user is logged in then we'll check to see which (if any) private
	// forums they are allowed to view and include them in the search.
	//
	// If not logged in we explicitly prevent searching of private forums
	//
	$auth_sql = '';
	$is_auth_ary = auth(AUTH_READ, AUTH_LIST_ALL, $userdata); 
	$ignore_forum_sql = '';
	while( list($key, $value) = each($is_auth_ary) )
	{
		if ( !$value['auth_read'] )
		{
			$ignore_forum_sql .= ( ( $ignore_forum_sql != '' ) ? ', ' : '' ) . $key;
		}
	}
	if ( $ignore_forum_sql != '' )
	{
		$auth_sql .= " p.forum_id NOT IN ($ignore_forum_sql)";
	}

	$sql = '';
	if ($auth_sql == '')
	{
		$sql = $basesql . $orderby;
	}
	else
	{
		$sql = $basesql . " AND " . $auth_sql . $orderby;
	}

        if (!($result = $db->sql_query($sql)))
        {
            message_die(CRITICAL_ERROR, "Error obtaining newest topics by user.", "", __LINE__, __FILE__, $sql);
       	}

	$rownumber = 0;
	while ($row = $db->sql_fetchrow($result))
	{
		$rownumber++;
		
		$post_time = create_date($board_config['default_dateformat'], $row['post_time'], $board_config['board_timezone']);
		$template->set_var ('row_number', $rownumber . '.');
		$template->set_var ('story_begin_href', '<a href="' . $_PHPBBB_CONF['phpbb_url'] . '/viewtopic.'.$phpEx.'?p=' . $row['post_id'] . '#' . $row['post_id'] . '">');
		$template->set_var ('story_title', $row['topic_title']);
		$template->set_var ('story_end_href', '</a>');
		$template->set_var ('story_date', $post_time);
		$template->parse ('forum_post_row', 'strow', true);
        }    
    
	if ($rownumber == 0)
	{
		$template->set_var ('forum_post_row', '<tr><td>' . $LANG_PHPBBB['no_user_posts'] . '</td></tr>');
	}

	$sql = "SELECT * from " . USERS_TABLE . " WHERE user_id=" . $uid;
        if (!($result = $db->sql_query($sql)))
        {
            message_die(CRITICAL_ERROR, "Error obtaining user info.", "", __LINE__, __FILE__, $sql);
       	}
        if (!($row = $db->sql_fetchrow($result)))
        {
            message_die(CRITICAL_ERROR, "Error obtaining user info.", "", __LINE__, __FILE__, $sql);
       	}

	$template->set_var('lang_number_forum_posts', $LANG_PHPBBB['total_num_posts']);
	$template->set_var('number_forum_posts', $row['user_posts']);

	$username = DB_getItem ($_TABLES['users'], 'username', "uid = $uid");
	$template->set_var('start_block_last10forumposts', COM_startBlock
	       ( $LANG_PHPBBB['last_10_posts_for_user'] . ' ' . $username));

	$template->set_var('lang_all_forum_postings_by', $LANG_PHPBBB['all_forum_postings_by'] . ' ' . $username);
	$template->set_var('forum_search_url', $_PHPBBB_CONF['phpbb_url'] . '/search.'.$phpEx.'?search_author='.$username);

	/* Display Instant Messaging Handles */
	$template->set_var('user_aim', $row['user_aim']);
	$template->set_var('user_icq', $row['user_icq']);
	$template->set_var('user_yim', $row['user_yim']);
	$template->set_var('user_msnm', $row['user_msnm']);
	$template->set_var('user_occupation', $row['user_occ']);
	$template->set_var('user_interests', $row['user_interests']);

	$template->set_var ('lang_aim', $LANG_PHPBBB['aim']);
	$template->set_var ('lang_icq', $LANG_PHPBBB['icq']);
	$template->set_var ('lang_yim', $LANG_PHPBBB['yim']);
	$template->set_var ('lang_msnm', $LANG_PHPBBB['msnm']);
	$template->set_var ('lang_occupation', $LANG_PHPBBB['occupation']);
	$template->set_var ('lang_interests', $LANG_PHPBBB['interests']);
}

?>
